%% Program for estimating data using 3-5 different methods.
%
% 1. Finds a relationship between Xdata and Fdata, either parameters of a 
%    discriminant function or by training a neural network
% 2. Simulates the system using the calculated parameters or a trained
%    neural network on a validating sample (Xval,Fval).
% 3. Compares the estimate Fval_e with output Fval.

% Author: Anders Fougner, anderfo@stud.ntnu.no
% Edited: Per Ferdinand Bach, perferdi@stud.ntnu.no

% $Revision: 2.5 $Date: 2009/03/28 14:20:00 $


function [F_e,Fdata,Fval_e,Fval,Ftest_e,Ftest,net] = estimationProgram(subject, feature, classifier, varargin)

global freq dir_setup tfArray

 for k=1:size(varargin,2) 
     switch char(varargin{k})
         case 'Order',
             order = varargin{k+1};
         case 'HiddenNodes',
             HiddenNodes = varargin{k+1};
         otherwise,
             %do nothing;
     end
 end

 net = 0;
 
% Retrieve data from files
if isnan(order)
    disp('Cannot find the input data, order not specified.');
    return;
end
path = [dir_setup 'input/'];

% Decompose the feature vector that was made in the combination.m
% file and then splice each feature in Xdata.
if(regexp(feature,'_'))
    numFeat = findstr('_',feature);
    for i=1:length(numFeat)+1
        if i==1
            combin{1} = feature(1:numFeat(1)-1);
        else
            try
                combin{i} = feature(numFeat(i-1)+1:numFeat(i)-1);
            catch
                combin{i} = feature(numFeat(i-1)+1:end);
            end
        end
    end
    Xdata = 0;
    for i=1:length(combin)
        load([path 'EMG' combin{i} '/XF_' combin{i} '_' subject '_v' order(1) '.mat']);
        Fdata = double(F); XTdata = X; clear F X;
        load([path 'EMG' combin{i} '/XF_' combin{i} '_' subject '_v' order(2) '.mat']);
        Fval = double(F); XTval = X; clear F X;
        load([path 'EMG' combin{i} '/XF_' combin{i} '_' subject '_v' order(3) '.mat']);
        Ftest = double(F); XTtest = X; clear F X;
        if size(XTdata,3)>=2
            for a=1:size(XTdata,3)
                if Xdata==0
                    Xdata = XTdata(:,:,a);
                    Xval  = XTval(:,:,a);
                    Xtest = XTtest(:,:,a);
                else
                    Xdata = [Xdata; XTdata(:,:,a)];
                    Xval  = [Xval; XTval(:,:,a)];
                    Xtest = [Xtest; XTtest(:,:,a)];
                end
            end
        else
            if Xdata==0
                Xdata = XTdata;
                Xval = XTval;
                Xtest = XTtest;
            else
                Xdata = [Xdata; XTdata];
                Xval = [Xval; XTval];
                Xtest = [Xtest; XTtest];
            end
        end
        clear XTdata XTval XTtest;
    end
else 
    load([path 'EMG' feature '/XF_' feature '_' subject '_v' order(1) '.mat']);
    Fdata = double(F); XTdata = X; clear F X;
    load([path 'EMG' feature '/XF_' feature '_' subject '_v' order(2) '.mat']);
    Fval = double(F); XTval = X; clear F X;
    load([path 'EMG' feature '/XF_' feature '_' subject '_v' order(3) '.mat']);
    Ftest = double(F); XTtest = X; clear F X;

    Xdata = 0;

    %If we have features with more than one coefficient for each timestep, we
    %need to construct the input data as multiple features.
    if size(XTdata,3)>=2
        for a=1:size(XTdata,3)
            if Xdata==0
                Xdata = XTdata(:,:,a);
                Xval  = XTval(:,:,a);
                Xtest = XTtest(:,:,a);
            else
                Xdata = [Xdata; XTdata(:,:,a)];
                Xval  = [Xval; XTval(:,:,a)];
                Xtest = [Xtest; XTtest(:,:,a)];
            end
        end
    else
        Xdata = XTdata;
        Xval = XTval;
        Xtest = XTtest;
    end
    HiddenNodes = HiddenNodes+(2*size(XTdata,3));
    clear XTdata XTval XTtest;
end
% Options
Tdata = size(Xdata,2);      % Tdata time steps for training data
Tval = size(Xval,2);        % Tval time steps for validation data
Ttest = size(Xtest,2);      % Ttest time steps for test data

Nx = size(Xdata,1);         % Nx signals
Nf = size(Fdata,1);         % Nf outputs signals

%% ESTIMATION
% Loop through some of the 5 different methods defined in init.m (tfArray)
%timeStart = cputime;
tic
switch char(classifier)
	case 'lf',
    	% 1st order estimation
        disp('Applying estimation with 1st order discriminant function...');
        [F_e(:,:),Fval_e(:,:),Ftest_e(:,:)] = firstOrderEstimation(Tdata,Tval,Ttest,Nx,Xdata,Fdata,Xval,Fval,Xtest,Ftest);
    case 'qf',
        % 2nd order estimation
        disp('Applying estimation with 2nd order discriminant function...');
        [F_e(:,:),Fval_e(:,:),Ftest_e(:,:)] = secondOrderEstimation(Tdata,Tval,Ttest,Nx,Xdata,Fdata,Xval,Fval,Xtest,Ftest);
	case 'nn',
        % Neural network
        if isnan(HiddenNodes)
            disp('You need to declare how many Hidden Nodes there are in the neural network');
            return;
        end
        disp('Applying neural network with ''tansig'' activation functions (may take some time!)...');
        disp(['Number of nodes: ' num2str(HiddenNodes)]);
        [F_e(:,:),Fval_e(:,:),Ftest_e(:,:),net] = neuralNetwork(Tdata,Tval,Ttest,Nx,Nf,Xdata,Fdata,Xval,Fval,['' deblank(tfArray(1,:)) ''],HiddenNodes,Xtest,Ftest,0);
    case 'rn',
        % Layered-recurrent neural network
        if isnan(HiddenNodes)
            disp('You need to declare how many Hidden Nodes there are in the neural network');
            return;
        end
        disp('Applying recurrent neural network with ''tansig'' activation functions (may take some time!)...');
        [F_e(:,:),Fval_e(:,:),Ftest_e(:,:),net] = recurrentNeuralNetwork(Tdata,Tval,Ttest,Nx,Nf,Xdata,Fdata,Xval,Fval,HiddenNodes,Xtest,Ftest,0);
    otherwise
        disp('Unknown method.')
end

estimationTime = toc;
%timeStop = cputime;
%estimationTime = timeStop-timeStart;
disp(['...finished after ' num2str(estimationTime/60) ' minutes.']);
