function [F_e,Fval_e,Ftest_e] = firstOrderEstimation(Tdata,Tval,Ttest,N,Xdata,Fdata,Xvaldata,Fvaldata,Xtestdata,Ftestdata)
% FIRSTORDERESTIMATION  Calculates first-order parameters W to estimate the
% function F from signals in Xdata using least-squares estimation.
% There are N signals and T time steps.
%
% For every time step we have
% F_e(X) = X'*W+ w0
%
% where
% X = (x1,x2,...,xN), W = (w1,w2,...,wN), w0 is a scalar treshold value
%
% We rewrite it as
% F_e(X) = X'*W+ W_0
%
% where
% X = (1,x1,x2,...,xN), W = (w0,w1,w2,...,wN)
%
% W is found such that we minimize V = 0.5*sum( (F_e(X) - F(X))^2 )
% where X and f(X) are known vectors.
%
% F_e, Fval_e and Ftest_e are then found using W.
%
% Finally, the function returns F_e, which is the estimate of F, 
% Fval_e, which is the estimate of Fval, and Ftest_e, which is
% the estimate of Ftest.
%
% See also SECONDORDERESTIMATION, NEURALNETWORK, MAKEDATA

% Anders Fougner, anderfo@stud.ntnu.no
% $Revision: 2.0 $Date: 2007/05/20 22:13:00 $

%% Estimation 1st order
% Performing least-squares estimation for each of N signals

% Adjust X
X = [ones(1,Tdata); Xdata]; % add 'ones' for calculating W_0 parameters

%Calculate the optimal W_big containing W, W_0
W = X'\Fdata';

% Calculate F_e(X), the estimate of F(x)
F_e = W'*X;

%% Simulate using validation data
% Adjust Xval
Xval = [ones(1,Tval); Xvaldata]; % add 'ones' for calculating W_0 parameters
% Calculate Fval_e(X), the estimate of Fval(x)
Fval_e = W'*Xval;

%% Simulate using test data
% Adjust Xtest
Xtest = [ones(1,Ttest); Xtestdata]; % add 'ones' for calculating W_0 parameters
% Calculate Fval_e(X), the estimate of Fval(x)
Ftest_e = W'*Xtest;