function [F_e,Fval_e,Ftest_e,net] = neuralNetwork(Tdata,Tval,Ttest,Nx,Nf,Xdata,Fdata,Xvaldata,Fvaldata,TfChoice,HiddenNodes,Xtestdata,Ftestdata,testNet)
% FIRSTORDERESTIMATION  Uses neural network theory to find a relation
% between a signal matrix X (for N signals and T time steps) and a function
% matrix F.
% Then it generates an estimate F_e of the function matrix F, using the
% neural net.
%
% Finally, the function returns F_e, which is the estimate of F, and
% according results for validation and test sets.
%
%
% See also MAKEDATA, FIRSTORDERESTIMATION, SECONDORDERESTIMATION

% Anders Fougner, anderfo@stud.ntnu.no
% $Revision: 2.0 $Date: 2007/05/20 22:13:00 $


% "Normalization/standardization" of output data
[Fdata2,FdataS] = mapminmax(Fdata);
[Fvaldata2,FvaldataS] = mapminmax(Fvaldata);
[Ftestdata2,FtestdataS] = mapminmax(Ftestdata);

% Choice of transfer function in the nodes/synapses of the NN

Tfs(1:HiddenNodes,1:Nf) = {TfChoice}; % tansig is default
disp(['Using ' num2str(HiddenNodes) ' nodes in the hidden layer.']);

% % Adjust X to 2nd degree
% X = Xdata;
% for k=Nx:-1:1
%     X = [X;(Xdata(1:k,1:Tdata).*Xdata(Nx-k+1:Nx,1:Tdata))];
% end
% % Adjust Xval in the same way as X was adjusted
% Xval = Xvaldata;
% for k=Nx:-1:1
%     Xval = [Xval;(Xvaldata(1:k,1:Tval).*Xvaldata(Nx-k+1:Nx,1:Tval))];
% end
% % Adjust Xtest in the same way as X was adjusted
% Xtest = Xtestdata;
% for k=Nx:-1:1
%     Xtest = [Xtest;(Xtestdata(1:k,1:Ttest).*Xtestdata(Nx-k+1:Nx,1:Ttest))];
% end

% Generate a neural network
if ~testNet
    net = newff(minmax(Xdata),minmax(Fdata),[HiddenNodes Nf],Tfs);

    % Make validation and testing structures
    VV.P = Xvaldata;
    VV.T = Fvaldata2;
end

TV.P = Xtestdata;
TV.T = Ftestdata2;

if ~testNet
    % Train the neural network, but do not print error messages
    net.trainParam.showWindow = false;
    net.trainParam.show = NaN;
    [net,tr]=train(net,Xdata,Fdata2,[],[],VV,TV);

    % Simulate Xdata and Xvaldata in the NN to estimate F and Fval
    F_e = sim(net,Xdata);
    Fval_e = sim(net,Xvaldata);
    Ftest_e = sim(net,Xtestdata);
else
    Ftest_e = sim(testNet,Xtestdata);
end

if ~testNet
    % "Denormalization/unitization" of output data
    F_e = mapminmax('reverse',F_e,FdataS);
    Fval_e = mapminmax('reverse',Fval_e,FvaldataS);
end
Ftest_e = mapminmax('reverse',Ftest_e,FtestdataS);

if testNet
    net = testNet;
end