function [F_e,Fval_e,Ftest_e,net] = recurrentNeuralNetwork(Tdata,Tval,Ttest,Nx,Nf,Xdata,Fdata,Xvaldata,Fvaldata,HiddenNodes,Xtestdata,Ftestdata,testNet)

% FIRSTORDERESTIMATION  Uses neural network theory to find a relation
% between a signal matrix X (for N signals and T time steps) and a function
% matrix F.
% Then it generates an estimate F_e of the function matrix F, using the
% neural net.
% Finally, the function returns F_e, which is the estimate of F, and
% according results for validation and test sets.

% Hojjat Rahmanpour, rahmanpo@stud.ntnu.no
% $Revision: 1.0 $Date: 2008/10/20 22:13:00 $


% "Normalization/standardization" of output data
[Fdata2,FdataS] = mapminmax(Fdata);
[Fvaldata2,FvaldataS] = mapminmax(Fvaldata);
[Ftestdata2,FtestdataS] = mapminmax(Ftestdata);
DataIn = minmax(Xdata);
DataOut = minmax(Fdata);
   
% GENERATE A LAYER-RECURRENT NEURAL NETWORK

%newlrn(P,T,[S1 S2...S(N-l)],{TF1 TF2...TFNl}, BTF,BLF,PF,IPF,OPF,DDF)
%P  R x Q1 matrix of Q1 sample R-element input vectors : input vector
%T  SN=4 x Q2 matrix of Q2 sample SN-element input vectors : target vectors
%(Output layer size SN is determined from T)
if ~testNet
    net = newlrn(DataIn, DataOut, [HiddenNodes Nf], {'tansig' 'tansig'});

    % Make validation and testing structures
    VV.P = Xvaldata;
    VV.T = Fvaldata2;
end

TV.P = Xtestdata;
TV.T = Ftestdata2;    
% Train the neural network, but do not print error messages
if ~testNet
    net.trainParam.showWindow = false;
    net.trainParam.epochs = 1000;
    net.trainParam.goal = 0.01;
    net.trainParam.show = NaN;
    [net,tr]=train(net,Xdata,Fdata2,[],[],VV,TV);

    % Simulate Xdata and Xvaldata in the NN to estimate F and Fval
    F_e = sim(net,Xdata);
    Fval_e = sim(net,Xvaldata);
    Ftest_e = sim(net,Xtestdata);
else
    Ftest_e = sim(testNet,Xtestdata);
end

if ~testNet
    % "Denormalization/unitization" of output data
    F_e = mapminmax('reverse',F_e,FdataS);
    Fval_e = mapminmax('reverse',Fval_e,FvaldataS);
end

Ftest_e = mapminmax('reverse',Ftest_e,FtestdataS);

if testNet
    net = testNet;
end