%% RESET WORKSPACE
clc, close all, clear all

%% SET PATH ENVIRONMENT
home = cd;
addbin = genpath([home '/bin/']);
addsrc = genpath([home '/src/']);
addpath(home,addbin,addsrc);

%% README
% We want the user to read the README file.
try
    %open README.txt
    disp('Please look through README..')
    %disp('Press any key to continue..')
    %pause
    %Should be able to alter a flag/value to define if this info should be
    %activated upon start.
catch
    %do nothing..
end

dato = datestr(now,'yyyymmdd');
%dato = '20090422'; %To work on previously created material
datostreng = datestr(now,'yyyymmdd_HHMMSS');
%% DEFINE VARIABLES
% Declare what variables to be global. Must be declared global in a
% function as well to have affect.

global EMG_freq freq VICON_freq smooth_const wname nMax numHistBins dir_setup tfArray Flowpass Klowpass post_filter_lp use_smooth_const;

% This file defines most of the variables that are possible to change.
EMG_freq = 1500; %EMG recorded at 1500Hz
VICON_freq = 60; % Frequency used to record the motion.
freq = 20; %The frequency we want the EMG to be scaled to.

features = {'aac','aav','ar','cc','ewc','ewcl','ewpc','ewpcl','hist','myop','nt','proc','var','wamp','wave','zc'}; %
subjects = {'sv','kjf','rlf'}; % Defining the different subjects.
dataset = {'v1','v2','v3'}; % Defining the different data sets.
classifier = {'lf','nn'}; % Defining the classifiers/mapping functions.

type = {'train','val','test'};  % Defining types of input to the classifiers.

%% FEATURE SUBSET
%Define if you want to find a subset.
% You will need a combinations.dat file with the setup of the features
% place it in the src/ folder.
% FEATURE_NAME1     FEATURE_NAME2
%       0                  0
%       0                  1
%       1                  0
%       1                  1

subset_filename = 'all_combinations_090421.dat'; %'combinations_090421c.dat';
feature_subset = 'true';
%% Simple calulcation
hpfilter_gain = 0.57;
hpfilter_frequency = 0.25;

scale = EMG_freq/freq; %scale x times.

%% Feature spec details
wname = 'sym4'; %haar,bior2.2,db2-3-4-5-6,coif1-2-3-4,sym4

nMax = wmaxlev(scale,wname); %  number of decompositions/coefficients
nMax = 5;
n = 2.^[1:nMax];
Frequency_blocks = scal2frq(n,wname,(1/scale));

numHistBins = 9; %num of bins

%% Directories
%FIKS DENNE, runCREATEDATA STRUCTURE MAA ENDREs
dir_setup = [home '/data/' dato '/EMG' num2str(freq) 'Hz/'];

%% VARIABLES for plotting features
chosenFeature_user = 1; %subject no.
chosenFeature_dataset = 2; %data set no.
chosenFeature_site = 1; % EMG signal no.
chosenFeature_combination = 'simple';

EMGsite =  {'Pronator' ...
            'Supinator' ...
            'FlexorDigitorum' ...
            'ExtensorDigitorum' ...
            'FlexorCarpiRadialis' ...
            'FlexorCarpiUlnaris' ...
            'ExtensorCarpiRadialis' ...
            'ExtensorCarpiUlnaris'};

% definerer omraade for  vise data
switch char(chosenFeature_combination)
    case 'simple',
        tmin = 18;
        tmax = 28;
    case 'combined',
        tmin = 75;
        tmax = 85;
end
raw = tmin*EMG_freq:tmax*EMG_freq;
range = tmin*freq:tmax*freq;
trange = tmin:1/freq:tmax; %vise tid, setter steglengde til 1/freq.

%% INITIALISATION FOR ESTIMATION
%Below should be defined in a singel file at root.

% Array of transfer function names for use in neural networks
tfArray(3:5,:) = char('tansig','logsig','purelin');
           
run = 50;   %Number of times we want to run estimation.
nodes = 10; % Number of nodes.
order = '123'; % Order to use as Training/Validation/Test.

nBestLaps = 10; % the n Best laps to find RMSE.

%% DEFINE NAMES OF ANGLES
nodeNo = 10; %should quite possibly be placed here.

angleROM = [120,120,170,20]; %degrees

% SIMPLE
angleNames = [ ...
    'finger flexion/extension';
    'wrist flexion/extension '; ...
    'pronation/supination    '; ...
    'radial/ulnar deviation  '];

aNshort = ['ffe';'wfe';'ps_';'rud'];

%COMBINED
angleCNames = [...
       'Finger flexion/extension and wrist flexion/extension'; ...
       'Finger flexion/extension and pronation/supination   '; ...
       'Finger flexion/extension and radial/ulnar deviation '; ...
       'Wrist flexion/extension and pronation/supination    '];    
aCNshort = ['ffewfe';'wfeps_';'psrud_';'rudps_'];
%% POST PROCESSING: FILTER SIGNAL

use_smooth_const=1;
smooth_const = 15; % num of samples to average over.

%% plotRMSE_xx
plotMappingF = 'lf';
plotMappingN = 'nn';
print_type = 3; %[train,val,test] data
persona = 1; % define what person we want to save rmse values for (.txt)?
%% plotLongTimeSeries
%define what angle we want to plot.
combo = 'true';
angle = 2; %number from variable angleNames
method = 2; %number from variable classifier
get_feature=10; % Feature number from variable features
get_combination=66; % Feature number from data file combination
get_subject=1;% Subject number from variable subjects
get_dataset=2; % Data set number from variable dataset
best_no = 1; % best_no 1:nBestLaps -> the one with the best RMSE.

tmin = 1; %280;
tmax = 100; %325; %370 %240